//
//  PongViewController.m
//  Pong
//
//  David J. Malan
//  Harvard University
//  dmalan@harvard.edu
//
//  Demonstrates Core Graphics via Pong.
//  Inspired by iTennis by Brandon Trebitowski.
//

#import "PongViewController.h"

#define VELOCITY 10.0f

@implementation PongViewController

@synthesize ball=_ball;
@synthesize labelLeft=_labelLeft;
@synthesize labelRight=_labelRight;
@synthesize paddleLeft=_paddleLeft;
@synthesize paddleRight=_paddleRight;
@synthesize paused=_paused;
@synthesize scoreLeft=_scoreLeft;
@synthesize scoreRight=_scoreRight;

- (void)awakeFromNib
{
    // initialize scores
    self.scoreLeft = 0;
    self.scoreRight = 0;
    
    // initialize ball's velocity
    _velocity = CGPointMake(VELOCITY, VELOCITY);
    
    // schedule gameplay
    [NSTimer scheduledTimerWithTimeInterval:0.05 target:self selector:@selector(play) userInfo:nil repeats:YES];
}

- (void)dealloc
{
    [_ball release];
    [_labelLeft release];
    [_labelRight release];
    [_paddleLeft release];
    [_paddleRight release];
    [super dealloc];
}

- (void)kickoff
{
    // update scores
    self.labelLeft.text = [NSString stringWithFormat:@"%u", self.scoreLeft];
    self.labelRight.text = [NSString stringWithFormat:@"%u", self.scoreRight];
    
    // center ball
    self.ball.center = CGPointMake(240.0f, 160.0f);
    
    // pause
    self.paused = YES;
}

- (void)play
{
    // check whether paused
    if (self.paused)
        return;
 
    // move ball
    self.ball.center = CGPointMake(self.ball.center.x + _velocity.x, self.ball.center.y + _velocity.y);
    
    // detect goals
    if (self.ball.center.x < 5) {
        self.scoreRight++;
        [self kickoff];
    }
    else if (self.view.bounds.size.width - 5 < self.ball.center.x) {
        self.scoreLeft++;
        [self kickoff];
    }
    
    // bounce off of top and bottom walls
    if (self.ball.center.y < 5 || self.view.bounds.size.height - 5 < self.ball.center.y) {
        _velocity.y = -_velocity.y;
    }

    // bounce off of left paddle
    if (CGRectIntersectsRect(self.ball.frame, self.paddleLeft.frame)) {
        if (self.paddleLeft.center.x < self.ball.center.x) {
            _velocity.x = -_velocity.x;
        }
    }
    
    // bounce off of right paddle
    if (CGRectIntersectsRect(self.ball.frame, self.paddleRight.frame)) {
        if (self.ball.center.x < self.paddleRight.center.x) {
            _velocity.x = -_velocity.x;
        }
    }
        
    // move opponent as ball approaches
    if (_velocity.x < 0) {

        // move down
        if (self.ball.center.y < self.paddleLeft.center.y) {
            self.paddleLeft.center = CGPointMake(self.paddleLeft.center.x, self.paddleLeft.center.y - 10.0f);
        }

        // move up
        else if (self.ball.center.y > self.paddleLeft.center.y) {
            self.paddleLeft.center = CGPointMake(self.paddleLeft.center.x, self.paddleLeft.center.y + 10.0f);
        }
    }
}

- (void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event
{
    // unpause if paused
    if (self.paused) {
        self.paused = NO;
    }
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    return (interfaceOrientation == UIInterfaceOrientationLandscapeRight);
}

- (void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event
{
    // follow user's finger vertically with left paddle
	UITouch *touch = [[event allTouches] anyObject];
	CGPoint location = [touch locationInView:self.view];
	self.paddleRight.center = CGPointMake(self.paddleRight.center.x, location.y);
}

- (void)viewDidLoad
{
    [super viewDidLoad];

    // play ball!
    [self kickoff];
}

- (void)viewDidUnload
{
    self.ball = nil;
    self.labelLeft = nil;
    self.labelRight = nil;
    self.paddleLeft = nil;
    self.paddleRight = nil;
    
    [super viewDidUnload];
}

@end
